// Copyright 2019 terrier989@gmail.com.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

import 'package:built_value/built_value.dart';
import 'package:built_value/serializer.dart';
import 'package:dent/core.dart';

part 'date.g.dart';

/// A date in the Gregorian calendar.
///
/// You can query values with [DateQuery].
class Date {
  static const Serializer serializer = _DateSerializer();

  final int year;
  final int month;
  final int day;

  const Date(this.year, this.month, this.day);

  factory Date.fromDateTime(DateTime dateTime) {
    return Date(dateTime.year, dateTime.month, dateTime.day);
  }

  @override
  int get hashCode =>
      year.hashCode ^ month.hashCode ^ day.hashCode ^ (day.hashCode << 4);

  @override
  bool operator ==(other) =>
      other is Date &&
      year == other.year &&
      month == other.month &&
      day == other.day;

  DateTime toDateTime() {
    return DateTime(year, month, day);
  }

  @override
  String toString() {
    final year = this.year.toString();
    final month = this.month.toString().padLeft(2, '0');
    final day = this.day.toString().padLeft(2, '0');
    return '$year-$month-$day';
  }

  static Date now() => Date.fromDateTime(DateTime.now());

  static Date parse(String s) {
    final i = s.indexOf('-');
    final j = s.indexOf('-', i + 1);
    final year = int.parse(s.substring(0, i));
    final month = int.parse(s.substring(i + 1, j));
    final day = int.parse(s.substring(j + 1));
    return Date(year, month, day);
  }
}

abstract class DateQuery
    implements PrimitiveQuery, Built<DateQuery, DateQueryBuilder> {
  static Serializer<DateQuery> get serializer => _$dateQuerySerializer;

  factory DateQuery([void Function(DateQueryBuilder b) updates]) = _$DateQuery;

  DateQuery._();
}

class _DateSerializer implements PrimitiveSerializer<Date> {
  const _DateSerializer();

  @override
  Iterable<Type> get types => const [Date];

  @override
  String get wireName => 'phone_number';

  @override
  Date deserialize(Serializers serializers, Object serialized,
      {FullType specifiedType = FullType.unspecified}) {
    if (serialized is String) {
      return Date.parse(serialized);
    } else {
      throw ArgumentError.value(serialized);
    }
  }

  @override
  Object serialize(Serializers serializers, Date object,
      {FullType specifiedType = FullType.unspecified}) {
    return '${object.year}-${object.month}-${object.day}';
  }
}
